@@ -24,19 +24,21 @@
 #
 ###########################################################################
 #
+import gzip
 import logging
 import os
 import re
 import shutil
-import socket
 import subprocess
 import tempfile
 from configparser import ConfigParser, ExtendedInterpolation
 from datetime import timedelta
-from typing import Optional
+from typing import Optional, Dict, List
+
+import pytest
+from filelock import FileLock
 
 from .certs import CertificateSpec, Credentials, TestCA
-from .ports import alloc_ports
 
 
 log = logging.getLogger(__name__)
@@ -58,9 +60,16 @@ CURL = os.path.join(TOP_PATH, 'src', 'curl')
 
 class EnvConfig:
 
-    def __init__(self):
+    def __init__(self, pytestconfig: Optional[pytest.Config] = None,
+                 testrun_uid=None,
+                 worker_id=None):
+        self.pytestconfig = pytestconfig
+        self.testrun_uid = testrun_uid
+        self.worker_id = worker_id if worker_id is not None else 'master'
         self.tests_dir = TESTS_HTTPD_PATH
-        self.gen_dir = os.path.join(self.tests_dir, 'gen')
+        self.gen_root = self.gen_dir = os.path.join(self.tests_dir, 'gen')
+        if self.worker_id != 'master':
+            self.gen_dir = os.path.join(self.gen_dir, self.worker_id)
         self.project_dir = os.path.dirname(os.path.dirname(self.tests_dir))
         self.build_dir = TOP_PATH
         self.config = DEF_CONFIG
@@ -113,21 +122,9 @@ class EnvConfig:
                     prot.lower() for prot in line[11:].split(' ')
                 }
 
-        self.ports = alloc_ports(port_specs={
-            'ftp': socket.SOCK_STREAM,
-            'ftps': socket.SOCK_STREAM,
-            'http': socket.SOCK_STREAM,
-            'https': socket.SOCK_STREAM,
-            'nghttpx_https': socket.SOCK_STREAM,
-            'proxy': socket.SOCK_STREAM,
-            'proxys': socket.SOCK_STREAM,
-            'h2proxys': socket.SOCK_STREAM,
-            'caddy': socket.SOCK_STREAM,
-            'caddys': socket.SOCK_STREAM,
-            'ws': socket.SOCK_STREAM,
-        })
+        self.ports = {}
+
         self.httpd = self.config['httpd']['httpd']
-        self.apachectl = self.config['httpd']['apachectl']
         self.apxs = self.config['httpd']['apxs']
         if len(self.apxs) == 0:
             self.apxs = None
@@ -147,6 +144,7 @@ class EnvConfig:
         self.expired_domain = f"expired.{self.tld}"
         self.cert_specs = [
             CertificateSpec(domains=[self.domain1, self.domain1brotli, 'localhost', '127.0.0.1'], key_type='rsa2048'),
+            CertificateSpec(name='domain1-no-ip', domains=[self.domain1, self.domain1brotli], key_type='rsa2048'),
             CertificateSpec(domains=[self.domain2], key_type='rsa2048'),
             CertificateSpec(domains=[self.ftp_domain], key_type='rsa2048'),
             CertificateSpec(domains=[self.proxy_domain, '127.0.0.1'], key_type='rsa2048'),
@@ -257,7 +255,6 @@ class EnvConfig:
 
     def is_complete(self) -> bool:
         return os.path.isfile(self.httpd) and \
-               os.path.isfile(self.apachectl) and \
                self.apxs is not None and \
                os.path.isfile(self.apxs)
 
@@ -266,8 +263,6 @@ class EnvConfig:
             return 'httpd not configured, see `--with-test-httpd=<path>`'
         if not os.path.isfile(self.httpd):
             return f'httpd ({self.httpd}) not found'
-        if not os.path.isfile(self.apachectl):
-            return f'apachectl ({self.apachectl}) not found'
         if self.apxs is None:
             return "command apxs not found (commonly provided in apache2-dev)"
         if not os.path.isfile(self.apxs):
@@ -290,9 +285,16 @@ class EnvConfig:
     def tcpdmp(self) -> Optional[str]:
         return self._tcpdump
 
+    def clear_locks(self):
+        ca_lock = os.path.join(self.gen_root, 'ca/ca.lock')
+        if os.path.exists(ca_lock):
+            os.remove(ca_lock)
+
 
 class Env:
 
+    SERVER_TIMEOUT = 30  # seconds to wait for server to come up/reload
+
     CONFIG = EnvConfig()
 
     @staticmethod
@@ -327,6 +329,13 @@ class Env:
     def curl_uses_lib(libname: str) -> bool:
         return libname.lower() in Env.CONFIG.curl_props['libs']
 
+    @staticmethod
+    def curl_uses_any_libs(libs: List[str]) -> bool:
+        for libname in libs:
+            if libname.lower() in Env.CONFIG.curl_props['libs']:
+                return True
+        return False
+
     @staticmethod
     def curl_uses_ossl_quic() -> bool:
         if Env.have_h3_curl():
@@ -385,6 +394,15 @@ class Env:
     def curl_is_debug() -> bool:
         return Env.CONFIG.curl_is_debug
 
+    @staticmethod
+    def curl_can_early_data() -> bool:
+        return Env.curl_uses_any_libs(['gnutls', 'wolfssl', 'quictls', 'openssl'])
+
+    @staticmethod
+    def curl_can_h3_early_data() -> bool:
+        return Env.curl_can_early_data() and \
+            Env.curl_uses_lib('ngtcp2')
+
     @staticmethod
     def have_h3() -> bool:
         return Env.have_h3_curl() and Env.have_h3_server()
@@ -425,7 +443,9 @@ class Env:
     def tcpdump() -> Optional[str]:
         return Env.CONFIG.tcpdmp
 
-    def __init__(self, pytestconfig=None):
+    def __init__(self, pytestconfig=None, env_config=None):
+        if env_config:
+            Env.CONFIG = env_config
         self._verbose = pytestconfig.option.verbose \
             if pytestconfig is not None else 0
         self._ca = None
@@ -433,11 +453,14 @@ class Env:
 
     def issue_certs(self):
         if self._ca is None:
-            ca_dir = os.path.join(self.CONFIG.gen_dir, 'ca')
-            self._ca = TestCA.create_root(name=self.CONFIG.tld,
-                                          store_dir=ca_dir,
-                                          key_type="rsa2048")
-        self._ca.issue_certs(self.CONFIG.cert_specs)
+            ca_dir = os.path.join(self.CONFIG.gen_root, 'ca')
+            os.makedirs(ca_dir, exist_ok=True)
+            lock_file = os.path.join(ca_dir, 'ca.lock')
+            with FileLock(lock_file):
+                self._ca = TestCA.create_root(name=self.CONFIG.tld,
+                                              store_dir=ca_dir,
+                                              key_type="rsa2048")
+                self._ca.issue_certs(self.CONFIG.cert_specs)
 
     def setup(self):
         os.makedirs(self.gen_dir, exist_ok=True)
@@ -466,6 +489,10 @@ class Env:
     def gen_dir(self) -> str:
         return self.CONFIG.gen_dir
 
+    @property
+    def gen_root(self) -> str:
+        return self.CONFIG.gen_root
+
     @property
     def project_dir(self) -> str:
         return self.CONFIG.project_dir
@@ -510,14 +537,25 @@ class Env:
     def expired_domain(self) -> str:
         return self.CONFIG.expired_domain
 
+    @property
+    def ports(self) -> Dict[str, int]:
+        return self.CONFIG.ports
+
+    def update_ports(self, ports: Dict[str, int]):
+        self.CONFIG.ports.update(ports)
+
     @property
     def http_port(self) -> int:
-        return self.CONFIG.ports['http']
+        return self.CONFIG.ports.get('http', 0)
 
     @property
     def https_port(self) -> int:
         return self.CONFIG.ports['https']
 
+    @property
+    def https_only_tcp_port(self) -> int:
+        return self.CONFIG.ports['https-tcp-only']
+
     @property
     def nghttpx_https_port(self) -> int:
         return self.CONFIG.ports['nghttpx_https']
@@ -578,10 +616,6 @@ class Env:
     def httpd(self) -> str:
         return self.CONFIG.httpd
 
-    @property
-    def apachectl(self) -> str:
-        return self.CONFIG.apachectl
-
     @property
     def apxs(self) -> str:
         return self.CONFIG.apxs
@@ -626,3 +660,24 @@ class Env:
                 i = int(fsize / line_length) + 1
                 fd.write(f"{i:09d}-{s}"[0:remain-1] + "\n")
         return fpath
+
+    def make_data_gzipbomb(self, indir: str, fname: str, fsize: int) -> str:
+        fpath = os.path.join(indir, fname)
+        gzpath = f'{fpath}.gz'
+        varpath = f'{fpath}.var'
+
+        with open(fpath, 'w') as fd:
+            fd.write('not what we are looking for!\n')
+        count = int(fsize / 1024)
+        zero1k = bytearray(1024)
+        with gzip.open(gzpath, 'wb') as fd:
+            for _ in range(count):
+                fd.write(zero1k)
+        with open(varpath, 'w') as fd:
+            fd.write(f'URI: {fname}\n')
+            fd.write('\n')
+            fd.write(f'URI: {fname}.gz\n')
+            fd.write('Content-Type: text/plain\n')
+            fd.write('Content-Encoding: x-gzip\n')
+            fd.write('\n')
+        return fpath
